
#include "pcrc.h"

#include <qapplication.h>
#include <qbuffer.h>
#include <qdatetime.h>
#include <qfont.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qsocketdevice.h>
#include <qsocketnotifier.h>
#include <qspinbox.h>
#include <qtimer.h>
#include <qvgroupbox.h>

#include <stdio.h>

#include "can.h"
#include "uss.h"
#include "eeprom.h"

pcrc::pcrc()
	: QSplitter( Qt::Vertical )
{
	QGroupBox	* group;
	QHBox	* box;
	
	group = new QVGroupBox( this );
	group->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum ) );
	group->setTitle( "server control" );
	
	box = new QHBox( group );
	
	sonar	=	new QPushButton( box );
	sonar->setText( "sonar" );
	sonar->setToggleButton( true );
	info	=	new QPushButton( box );
	info->setText( "inf" );
	addr	=	new QLineEdit( box );
	addr->setText( "192.168.13.13" );
	port	=	new QSpinBox( box );
	port->setRange( 0x0000, 0xffff );
	port->setValue( 42135 );
	reset	=	new QPushButton( box );
	reset->setText( "rst" );
	
	stat	=	new QLabel( group );
	
	eerom	= new eeprom( group );
	
	can	=	new CAN( this );
	
	uss	=	new USS( 0 );
	
	ntdv	=	new QSocketDevice( QSocketDevice::Datagram );
	QSocketNotifier * notify = new QSocketNotifier( ntdv->socket(), QSocketNotifier::Read, this );
	notify->setEnabled( true );
	
	connect( notify, SIGNAL( activated( int ) ), this, SLOT( readSocket() ) );
	connect( sonar, SIGNAL( toggled( bool ) ), uss, SLOT( setShown( bool ) ) );
	connect( info, SIGNAL( clicked() ), this, SLOT( queryInfo() ) );
	connect( reset, SIGNAL( clicked() ), this, SLOT( resetDevice() ) );
	
	connect( can,	SIGNAL( sigSnd( msg ) ), this, SLOT( sendCanMessage( msg ) ) );
	//connect( uss,	SIGNAL( sigSnd( msg ) ), this, SLOT( sendCanMessage( msg ) ) );
	queryInfo();
	
	connect( eerom, SIGNAL( transmit( byte *, int ) ),
		 this, SLOT( sendEE( byte *, int ) ) );

	QValueList<int> sz = sizes();
	sz[1] = sz[1] + sz[0];
	sz[0] = 0;
	setSizes( sz );
}

pcrc::~pcrc()
{
	if ( ntdv )
		delete ntdv;
}

void	pcrc::queryInfo()
{
	stat->setText( "sending info request ... " );
	QBuffer buffer;
	buffer.open( IO_WriteOnly );
	buffer.putch( 0x00 );
	buffer.putch( 0x00 );
	buffer.close();
	QHostAddress adr;
	adr.setAddress( addr->text() );
	ntdv->writeBlock( buffer.buffer().data(), buffer.buffer().size(), adr, port->value() );	
}

void	pcrc::resetDevice()
{
	stat->setText( "reset command send !" );
	QBuffer buffer;
	buffer.open( IO_WriteOnly );
	buffer.putch( 0xff );
	buffer.putch( 0x31 );
	buffer.putch( 0xca );
	buffer.close();
	QHostAddress adr;
	adr.setAddress( addr->text() );
	ntdv->writeBlock( buffer.buffer().data(), buffer.buffer().size(), adr, port->value() );	
}

void	pcrc::sendEE( byte * buffer, int len )
{
	QHostAddress adr;
	adr.setAddress( addr->text() );
	
	ntdv->writeBlock( (const char *) buffer, len, adr, port->value() );
	
	printf( "out\t" );
	for ( int i = 0; i < len; i++ )
		printf( "%x ", buffer[ i ] );
	printf( "\n" );
}

void	pcrc::sendCanMessage( msg m )
{
	stat->setText( "can command sent ..." );
	
	byte	buffer[ 0x100 ];

	buffer[ 0 ] = 0x31;
	buffer[ 1 ] = m.id;
	buffer[ 2 ] = 0xa8;
	buffer[ 3 ] = m.cmd;
	buffer[ 4 ] = m.par;
	buffer[ 5 ] = ( m.len | ( m.rtr ? 1 << 6 : 0x00 ) );
	for ( int i = 0; i < 8; i++ )
		buffer[ 6 + i ] = ( m.dat[ i ] );
	
	QHostAddress adr;
	adr.setAddress( addr->text() );
	
	ntdv->writeBlock( (const char *) buffer, 15, adr, port->value() );	
	
	printf( "out\t" );
	for ( int i = 0; i < 15; i++ )
		printf( "%x ", buffer[ i ] );
	printf( "\n" );
}

void	pcrc::readSocket()
{
	byte	buffer[ 0x10000 ];
	int	cnt	=	ntdv->readBlock( (char *) buffer, 0xffff );
	printf( "in\t" );
	for ( int i = 0; i < cnt; i++ )
		printf( "%x ", buffer[ i ] );
	printf( "\n" );
	
	if ( cnt > 0 )
	{
		if ( buffer[ 0 ] == 0x00 )
		{
			if ( buffer[ 1 ] == 0x01 )
				stat->setText( "server up and running" );
			else
				stat->setText( "server down" );
		}
		else if ( buffer[ 0 ] == 0x31 )
		{
			stat->setText( "can message received" );
			msg m( & buffer[ 1 ] );
			can->rcv( m );
			uss->rcv( m );
		}
		else if ( buffer[ 0 ] == 0x81 )
		{
			stat->setText( "eeprom read command received" );
			eerom->receive( buffer, cnt );
		}
		else if ( buffer[ 0 ] == 0x82 )
		{
			stat->setText( "eeprom write command received" );
			eerom->receive( buffer, cnt );
		}
		else
		{
			stat->setText( QString( "unknown code (%1) received" ).arg( buffer[ 0 ] ) );
		}
	}
	else
	{
		stat->setText( "empty message received" );
	}
}

int	main(	int argc,	char * argv[]	)
{
	ASSERT( sizeof( msg ) == 13 );
	
	qDebug( QString::number( sizeof( msg ) ) );
	QApplication	a( argc, argv );
	a.setFont( QFont( "xyz", 20 ) );
	
	pcrc		p;
	p.showMaximized();
	a.setMainWidget( &p );
	
	return a.exec();
}
